Diplomat = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local options = { 
    type='group',
    args = {
        setfac = {
            type = 'text',
            name = 'Current faction',
            desc = 'Associate the current zone with a faction, "none" for no bar',
            get = "GetZone",
            set = "SetZone",
            usage = "<Your faction>",
	    message = "%s is now associated with %s",
        },
	list = {
	    type = 'execute',
	    desc = "List ID's and associated factions",
	    name = "List ID's",
	    func = "ListIDs",
	},
    },
}

Diplomat:RegisterDB("DiplomatDB", "DiplomatDBPC")

Diplomat:RegisterDefaults("profile", {
	ZoneToFaction = {}
} )

Diplomat:RegisterChatCommand({"/diplomat", "/dip"}, options)

function Diplomat:OnEnable()
    self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
end

function Diplomat:OnDisable()
    self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
end

function Diplomat:ScanFactions()
    local factions = {}
    local name, isHeader, isCollapsed
    factions["none"] = 0
    
    -- expand the collapsed items, so we can scan through them all
    for i = 0, GetNumFactions() do
        _, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
        if isHeader and isCollapsed then
            ExpandFactionHeader(i)
        end
    end
    
    for i = 0, GetNumFactions() do
        name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
        if not isHeader and name ~= nil then
            factions[string.lower(name)] = i
        end
    end
    self.db.profile.factions = factions
end

function Diplomat:ZONE_CHANGED_NEW_AREA()
    self:ScanFactions()
    
    local faction = self:GetZone()
    if faction == nil then
        self:Print("Zoned into "..GetZoneText()..", no faction set yet")
        self:Print("To set a faction to this zone type: /diplomat setfac <faction>")
        SetWatchedFactionIndex(0)
    else
        faction = string.lower(faction)
        self:Print("Zoned into "..GetZoneText()..", with faction "..faction)
        if(self.db.profile.factions[faction] ~= nil) then
            SetWatchedFactionIndex(self.db.profile.factions[faction])
        else
            self:Print("Faction is not in list, assuming none")
            SetWatchedFactionIndex(0)
        end
    end
end

function Diplomat:GetZone() return self.db.profile.ZoneToFaction[GetZoneText()] end

function Diplomat:SetZone(value)
    if(self.db.profile.factions[string.lower(value)] ~= nil) then
        self.db.profile.ZoneToFaction[GetZoneText()] = string.lower(value)
        SetWatchedFactionIndex(self.db.profile.factions[string.lower(value)])
    else
        self:Print("Faction is not in list")
    end
end

function Diplomat:ListIDs()
    local name, isHeader
    
    -- expand the collapsed items, so we can scan through them all
    for i = 0, GetNumFactions() do
        _, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
        if isHeader and isCollapsed then
            ExpandFactionHeader(i)
        end
    end
    
    for i = 0, GetNumFactions() do
        name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
        if not isHeader and name ~= nil then
            self:Print("Faction "..name.." with ID "..RepIndex)
        end
    end
end
